"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
class PricingTier {
    constructor(pricingTierId) {
        this.pricingTierId = pricingTierId;
    }
    //
    // Static methods
    //
    static create(pricingTier, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/pricing-tiers",
                    data: pricingTier,
                    options,
                });
                return new PricingTier(response.pricingTierId);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/pricing-tiers/${pricingTierId}`,
                    options,
                });
                return new PricingTier(response.pricingTierId);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/pricing-tiers/${pricingTierId}`,
                    options,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPricingTiers(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/pricing-tiers",
                    params: listOptions,
                    options,
                });
                return response.map((pricingTier) => new PricingTier(pricingTier.pricingTierId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPricingTiersForTenant(tenantId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/tenants/${tenantId}/pricing-tiers`,
                    params: listOptions,
                    options,
                });
                return response.map((pricingTier) => new PricingTier(pricingTier.pricingTierId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPricingTierToTenant(tenantId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static removePricingTierFromTenant(tenantId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static listPricingTiersForUser(userId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}/pricing-tiers`,
                    params: listOptions,
                    options,
                });
                return response.map((pricingTier) => new PricingTier(pricingTier.pricingTierId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPricingTierToUser(userId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removePricingTierFromUser(userId, pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    // Instance methods
    listFeatures(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForPricingTier(this.pricingTierId, listOptions, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToPricingTier(this.pricingTierId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromPricingTier(this.pricingTierId, featureId, options);
        });
    }
    hasFeature(featureId, context = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: featureId, subject: { objectType: ObjectType_1.ObjectType.PricingTier, objectId: this.pricingTierId }, context: context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.PricingTier;
    }
    getObjectId() {
        return this.pricingTierId;
    }
}
exports.default = PricingTier;
