import Permission from "./Permission";
import { WarrantRequestOptions } from "../types/WarrantRequestOptions";
import { ListPermissionOptions } from "../types/Permission";
import { CreateRoleParams, ListRoleOptions, UpdateRoleParams } from "../types/Role";
import Warrant, { PolicyContext, WarrantObject } from "../types/Warrant";
export default class Role implements WarrantObject {
    roleId: string;
    name?: string;
    description?: string;
    constructor(roleId: string, name?: string, description?: string);
    static create(role: CreateRoleParams, options?: WarrantRequestOptions): Promise<Role>;
    static get(roleId: string, options?: WarrantRequestOptions): Promise<Role>;
    static update(roleId: string, role: UpdateRoleParams, options?: WarrantRequestOptions): Promise<Role>;
    static delete(roleId: string, options?: WarrantRequestOptions): Promise<void>;
    static listRoles(listOptions?: ListRoleOptions, options?: WarrantRequestOptions): Promise<Role[]>;
    static listRolesForUser(userId: string, listOptions?: ListRoleOptions, options?: WarrantRequestOptions): Promise<Role[]>;
    static assignRoleToUser(userId: string, roleId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    static removeRoleFromUser(userId: string, roleId: string, options?: WarrantRequestOptions): Promise<void>;
    listPermissions(listOptions?: ListPermissionOptions, options?: WarrantRequestOptions): Promise<Permission[]>;
    assignPermission(permissionId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePermission(permissionId: string, options?: WarrantRequestOptions): Promise<void>;
    hasPermission(permissionId: string, context?: PolicyContext, options?: WarrantRequestOptions): Promise<boolean>;
    getObjectType(): string;
    getObjectId(): string;
}
