import Feature from "./Feature";
import PricingTier from "./PricingTier";
import User from "./User";
import Warrant from "./WarrantModule";
import { ListFeatureOptions } from "../types/Feature";
import { WarrantRequestOptions } from "../types/WarrantRequestOptions";
import { ListPricingTierOptions } from "../types/PricingTier";
import { CreateTenantParams, ListTenantOptions, UpdateTenantParams } from "../types/Tenant";
import { ListUserOptions } from "../types/User";
import { PolicyContext, WarrantObject } from "../types/Warrant";
export default class Tenant implements WarrantObject {
    tenantId: string;
    name?: string;
    constructor(tenantId: string, name?: string);
    static create(tenant?: CreateTenantParams, options?: WarrantRequestOptions): Promise<Tenant>;
    static batchCreate(tenants: CreateTenantParams[], options?: WarrantRequestOptions): Promise<Tenant[]>;
    static get(tenantId: string, options?: WarrantRequestOptions): Promise<Tenant>;
    static update(tenantId: string, tenant: UpdateTenantParams, options?: WarrantRequestOptions): Promise<Tenant>;
    static delete(tenantId: string, options?: WarrantRequestOptions): Promise<void>;
    static listTenants(listOptions?: ListTenantOptions, options?: WarrantRequestOptions): Promise<Tenant[]>;
    static listTenantsForUser(userId: string, listOptions?: ListTenantOptions, options?: WarrantRequestOptions): Promise<Tenant[]>;
    listUsers(listOptions?: ListUserOptions, options?: WarrantRequestOptions): Promise<User[]>;
    assignUser(userId: string, role: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeUser(userId: string, role: string, options?: WarrantRequestOptions): Promise<void>;
    listPricingTiers(listOptions?: ListPricingTierOptions, options?: WarrantRequestOptions): Promise<PricingTier[]>;
    assignPricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<void>;
    listFeatures(listOptions?: ListFeatureOptions, options?: WarrantRequestOptions): Promise<Feature[]>;
    assignFeature(featureId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeFeature(featureId: string, options?: WarrantRequestOptions): Promise<void>;
    hasFeature(featureId: string, context?: PolicyContext, options?: WarrantRequestOptions): Promise<boolean>;
    getObjectType(): string;
    getObjectId(): string;
}
