"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Warrant_1 = require("./Warrant");
class Query {
    constructor(selectClause, forClause = {}, whereClause = {}) {
        this.selectClause = selectClause;
        this.forClause = forClause;
        this.whereClause = whereClause;
    }
    static selectWarrants() {
        return new Query("warrants");
    }
    static selectExplicitWarrants() {
        return new Query("explicit warrants");
    }
    for(forClause) {
        this.forClause = forClause;
        return this;
    }
    where(whereClause) {
        this.whereClause = whereClause;
        return this;
    }
    toObject() {
        let queryParams = {};
        queryParams["select"] = this.selectClause;
        if (Object.keys(this.forClause).length > 0) {
            let forParamsString = "";
            for (const [key, value] of Object.entries(this.forClause)) {
                switch (key) {
                    case 'object':
                        if (Warrant_1.isWarrantObject(value)) {
                            forParamsString += `${key}=${value.getObjectType()}:${value.getObjectId()},`;
                        }
                        else {
                            forParamsString += `${key}=${value.objectType}:${value.objectId},`;
                        }
                        break;
                    case 'relation':
                        forParamsString += `${key}=${value},`;
                        break;
                    case 'subject':
                        if (Warrant_1.isSubject(value)) {
                            if (value.relation) {
                                forParamsString += `${key}=${value.objectType}:${value.objectId}#${value.relation},`;
                            }
                            else {
                                forParamsString += `${key}=${value.objectType}:${value.objectId},`;
                            }
                        }
                        else {
                            forParamsString += `${key}=${value.getObjectType()}:${value.getObjectId()},`;
                        }
                        break;
                    case 'context':
                        let contextString = '';
                        for (const [contextKey, contextValue] of Object.entries(value)) {
                            contextString += `${contextKey}=${contextValue} `;
                        }
                        contextString = contextString.slice(0, -1);
                        forParamsString += `${key}=${contextString},`;
                        break;
                }
            }
            forParamsString = forParamsString.slice(0, -1);
            queryParams["for"] = forParamsString;
        }
        if (Object.keys(this.whereClause).length > 0) {
            let whereParamsString = "";
            for (const [key, value] of Object.entries(this.whereClause)) {
                switch (key) {
                    case 'object':
                        if (Warrant_1.isWarrantObject(value)) {
                            whereParamsString += `${key}=${value.getObjectType()}:${value.getObjectId()},`;
                        }
                        else {
                            whereParamsString += `${key}=${value.objectType}:${value.objectId},`;
                        }
                        break;
                    case 'relation':
                        whereParamsString += `${key}=${value},`;
                        break;
                    case 'subject':
                        if (Warrant_1.isSubject(value)) {
                            if (value.relation) {
                                whereParamsString += `${key}=${value.objectType}:${value.objectId}#${value.relation},`;
                            }
                            else {
                                whereParamsString += `${key}=${value.objectType}:${value.objectId},`;
                            }
                        }
                        else {
                            whereParamsString += `${key}=${value.getObjectType()}:${value.getObjectId()},`;
                        }
                        break;
                    case 'context':
                        let contextString = '';
                        for (const [contextKey, contextValue] of Object.entries(value)) {
                            contextString += `${contextKey}=${contextValue} `;
                        }
                        contextString = contextString.slice(0, -1);
                        whereParamsString += `${key}=${contextString},`;
                        break;
                }
            }
            whereParamsString = whereParamsString.slice(0, -1);
            queryParams["where"] = whereParamsString;
        }
        return queryParams;
    }
}
exports.default = Query;
